const c = require('ansi-colors')

const { createCache } = require('./cache')

const { NODE_ENV } = process.env

const requests = {}
const errors = {}
const cache = createCache('presta-load-cache')

function log (str) {
  if (NODE_ENV !== 'test') console.log(str)
}

function prime (key, value, duration) {
  cache.set(key, value, duration)
}

function loadError (key, e) {
  log(`\n  ${c.red('error')} load { ${key} }\n\n${e}\n`)
  errors[key] = e
  delete requests[key]
}

function load (loader, { key, duration }) {
  let value = cache.get(key)

  if (!value && !errors[key]) {
    // try/catch required for sync loaders
    try {
      requests[key] = loader()

      requests[key]
        .then(value => {
          cache.set(key, value, duration)
          delete requests[key]
        })
        // catch async errors
        .catch(e => loadError(key, e))
    } catch (e) {
      loadError(key, e)
    }
  }

  delete errors[key]

  return value
}

async function flush (run, data = {}) {
  const content = run()

  if (Object.keys(requests).length) {
    await Promise.allSettled(Object.values(requests))
    return flush(run, data)
  }

  return { content, data: cache.dump() }
}

module.exports = {
  prime,
  load,
  flush
}
